### Persephone - a socio-economic-ecological model of European agricultural landscapes.
###
### This file is responsible for managing the crop growth modules.
###

#XXX not sure whether it makes sense to have this as an agent type,
# or perhaps better a grid property?
"""
    FarmPlot

This represents one field, i.e. a collection of pixels with the same management.
This is the spatial unit with which the crop growth model and the farm model work.
"""
@agent FarmPlot NoSpaceAgent begin
    pixels::Vector{Tuple{Int64, Int64}}
    croptype::String
    height::Float64
    biomass::Float64
    #TODO
end

"""
    stepagent!(farmplot, model)

Update a farm plot by one day.
"""
function stepagent!(farmplot::FarmPlot, model::AgentBasedModel)
    #TODO
end

"""
    initfields!(model)

Initialise the model with its farm plots.
"""
function initfields!(model::AgentBasedModel)
    n = 0
    convertid = Dict{Int64,Int64}()
    width, height = size(model.landscape)
    for x in 1:width
        for y in 1:height
            # for each pixel, we need to extract the field ID given by the map input file,
            # and convert it into the internal object ID used by Agents.jl, creating a new
            # agent object if necessary
            rawid = model.landscape[x,y].fieldid
            (ismissing(rawid)) && continue
            if rawid in keys(convertid)
                objectid = convertid[rawid]
                model.landscape[x,y].fieldid = objectid
                push!(model[objectid].pixels, (x,y))
            else
                fp = add_agent!(FarmPlot, model, [(x,y)], "fallow", 0.0, 0.0)
                model.landscape[x,y].fieldid = fp.id
                convertid[rawid] = fp.id
                n += 1
            end
        end
    end
    @debug "Initialised $n farm plots."
end

#XXX only needed during development, can be deleted again?
function countfields(model::AgentBasedModel)
    ids::Vector{Int64} = []
    width, height = size(model.landscape)
    for x in 1:width
        for y in 1:height
            i = model.landscape[x,y].fieldid
            (ismissing(i)) && continue
            push!(ids, i)
        end
    end
    ids |> sort! |> unique! |> length
end

"""
    averagefieldsize(model)

Calculate the average field size in hectares for the model landscape.
"""
function averagefieldsize(model::AgentBasedModel)
    conversionfactor = 100 #our pixels are currently 10x10m, so 100 pixels per hectare
    sizes::Vector{Float64} = []
    for a in allagents(model)
        (typeof(a) != FarmPlot) && continue
        push!(sizes, size(a.pixels)[1]/conversionfactor)
    end
    round(sum(sizes)/size(sizes)[1], digits=2)
    #sizes
end
